/*
 
     File: PPPPPPPP.cpp
 Abstract: Audio Unit class implementation.
  Version: 1.0.1
 
    Template by Alex Kenis 2014
*/

#include "PPPPPPPP.h" // Include our main project header file and all the *.h files included there

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Register our AuduiUnit so the host can find it
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// COMPONENT_ENTRY(PPPPPPPP) deprecated on MacOS X 10.7 see TN2276

AUDIOCOMPONENT_ENTRY(AUBaseFactory, PPPPPPPP) // New macro... for now

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	PPPPPPPP::PPPPPPPP
//      Create our global controls and set them to their default values from the kDefault
//      that we set up in our header
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PPPPPPPP::PPPPPPPP(AudioUnit component)
	: AUEffectBase(component)
{
    
    // This method, defined in the AUBase superclass, ensures that the required audio unit
	//  elements are created and initialized.
	CreateElements();
    
    // Invokes the use of an STL vector for parameter access.
	//  See AUBase/AUScopeElement.cpp
	Globals()->UseIndexedParameters(kNumberOfParameters);
    
    // During instantiation, sets up the parameters according to their defaults.
	//	The parameter defaults should correspond to the settings for the default preset.
    //  These calls have the effect both of defining the parameters for the first time
	//  and assigning their initial values
	SetParameter(kParam_One, kDefaultValue_ParamOne ); //Gain default
    SetParameter(kParam_Two, kDefaultValue_ParamTwo ); //Tone default
    SetParameter(kParam_Three, kDefaultValue_ParamThree ); //Vol default
        
#if AU_DEBUG_DISPATCHER
	mDebugDispatcher = new AUDebugDispatcher (this);
#endif
	
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	PPPPPPPP::Initialize
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
OSStatus			PPPPPPPP::Initialize()
{
	OSStatus result = AUEffectBase::Initialize();
	
	if(result == noErr )
	{
        // other inits go here, we don't have any for this plugin
	}
	
	return result;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	PPPPPPPP::GetParameterValueStrings
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Provides the strings for generic view (if any)
OSStatus			PPPPPPPP::GetParameterValueStrings(AudioUnitScope		inScope,
                                                                AudioUnitParameterID	inParameterID,
                                                                CFArrayRef *		outStrings)
{
        
    return kAudioUnitErr_InvalidProperty;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	PPPPPPPP::GetParameterInfo
//      Set info on our controls for the generic ui: unit type, behavior, etc
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Called by the audio unit's view; provides information needed for the view to display the
//  audio unit's parameters
OSStatus			PPPPPPPP::GetParameterInfo(AudioUnitScope		inScope,
                                                        AudioUnitParameterID	inParameterID,
                                                        AudioUnitParameterInfo	&outParameterInfo )
{
	OSStatus result = noErr;

    // Adds two flags to all parameters for the audio unit, indicating to the host application
	// that it should consider all the audio units parameters to be readable and writable.
	outParameterInfo.flags = 	kAudioUnitParameterFlag_IsWritable
						|		kAudioUnitParameterFlag_IsReadable;
    // All three parameters for this audio unit are in the "global" scope.
    if (inScope == kAudioUnitScope_Global) {
        switch(inParameterID)
        {
                // Gain
            case kParam_One:
                // Invoked when the view needs information for the kParameterOneName (gain)
                // parameter; defines how to represent this parameter in the user interface.
                AUBase::FillInParameterName (outParameterInfo, kParameterOneName, false);
                outParameterInfo.unit = kAudioUnitParameterUnit_LinearGain; // display unit
                outParameterInfo.minValue = 1; // minimum gain multiplier
                outParameterInfo.maxValue = 100; // maximum gain multiplier
                outParameterInfo.defaultValue = kDefaultValue_ParamOne;
                outParameterInfo.flags += kAudioUnitParameterFlag_DisplayLogarithmic; // make control log
                break;
                
                // Tone
            case kParam_Two:
                AUBase::FillInParameterName (outParameterInfo, kParameterTwoName, false);
                outParameterInfo.unit = kAudioUnitParameterUnit_Hertz; // display unit
                outParameterInfo.minValue = 500; // minimum (in Hz)
                outParameterInfo.maxValue = 999; // maximum (in Hz)
                outParameterInfo.defaultValue = kDefaultValue_ParamTwo;
                break;
                
                // Volume
            case kParam_Three:
                AUBase::FillInParameterName (outParameterInfo, kParameterThreeName, false);
                outParameterInfo.unit = kAudioUnitParameterUnit_Generic; // display unit
                outParameterInfo.minValue = 0.01; //minimum volume multiplier
                outParameterInfo.maxValue = 1; //maximum volume multiplier
                outParameterInfo.defaultValue = kDefaultValue_ParamThree;
                outParameterInfo.flags += kAudioUnitParameterFlag_DisplayLogarithmic; // make control log
                break;
                
            default:
                result = kAudioUnitErr_InvalidParameter;
                break;
            }
	} else {
        result = kAudioUnitErr_InvalidParameter;
    }

	return result;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	PPPPPPPP::GetPropertyInfo
//      info on our cocoa UI custom property
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
OSStatus			PPPPPPPP::GetPropertyInfo (AudioUnitPropertyID	inID,
                                                        AudioUnitScope		inScope,
                                                        AudioUnitElement	inElement,
                                                        UInt32 &		outDataSize,
                                                        Boolean &		outWritable)
{
	if (inScope == kAudioUnitScope_Global) 
	{
		switch (inID) 
		{
			case kAudioUnitProperty_CocoaUI:
				outWritable = false;
				outDataSize = sizeof (AudioUnitCocoaViewInfo);
				return noErr;
					
		}
	}

	return AUEffectBase::GetPropertyInfo (inID, inScope, inElement, outDataSize, outWritable);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	PPPPPPPP::GetProperty
//      info on our cocoa UI
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
OSStatus			PPPPPPPP::GetProperty(	AudioUnitPropertyID inID,
															AudioUnitScope 		inScope,
															AudioUnitElement 	inElement,
															void *				outData )
{
	if (inScope == kAudioUnitScope_Global) 
	{
		switch (inID) 
		{
                // This property allows the host application to find the UI associated with this AudioUnit
			case kAudioUnitProperty_CocoaUI:
			{
				// Look for a resource in the main bundle by name and type.
				CFBundleRef bundle = CFBundleGetBundleWithIdentifier( CFSTR("com.audiounit.PPPPPPPP") );
				
				if (bundle == NULL) return fnfErr;
                
				CFURLRef bundleURL = CFBundleCopyResourceURL( bundle, 
                    CFSTR("PPPPPPPP_CocoaViewFactory"), // this is the name of the cocoa bundle as specified in the plist
                    CFSTR("bundle"), // this is the extension of the cocoa bundle
                    NULL);
                
                if (bundleURL == NULL) return fnfErr;

				AudioUnitCocoaViewInfo cocoaInfo;
				cocoaInfo.mCocoaAUViewBundleLocation = bundleURL;
				cocoaInfo.mCocoaAUViewClass[0] = CFStringCreateWithCString(NULL, "PPPPPPPP_CocoaViewFactory", kCFStringEncodingUTF8); // name of the main class that implements
				
				*((AudioUnitCocoaViewInfo *)outData) = cocoaInfo;
				
				return noErr;
			}
		}
	}
    // handles the standard properties
	return AUEffectBase::GetProperty (inID, inScope, inElement, outData);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	PPPPPPPP::PPPPPPPPKernel()
//      Clean up our kernel
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// This is the constructor for the PPPPPPPPKernel helper class, which holds the DSP code
//  for the audio unit: one kernel object gets built for each channel in the audio unit.
#pragma mark ____PPPPPPPPEffectKernel

PPPPPPPPKernel::PPPPPPPPKernel(AUEffectBase *inAudioUnit )
: AUKernelBase(inAudioUnit)
{
	Reset();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	AmpTemplateKernel::~AmpTemplateKernel()
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PPPPPPPPKernel::~PPPPPPPPKernel( )
{
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	AmpTemplateKernel::Reset()
//
//		It's very important to fully reset all state variables to their
//		initial settings here.  For delay/reverb effects, the delay buffers must
//		also be cleared here. Reinitialize the variables that were initialized on
//      instantiation of the kernel object.
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void		PPPPPPPPKernel::Reset()
{
	// forces  initial param calculation for our host
    mLastGain = -1.1;
    mLastTone = -1.1;
    mLastVol = -1.1;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	PPPPPPPP::PPPPPPPPKernel::Process
//      This is our process function.  The DSP number-crunching is done here.
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void PPPPPPPPKernel::Process(	const Float32 	*inSourceP, // buffer input pointer
                                                    Float32		 	*inDestP, // buffer output pointer
                                                    UInt32 			inFramesToProcess, // number of samples in our buffer
                                                    UInt32			inNumChannels, // always 1
                                                    bool			&ioSilence ) // don't process if silent
{
  // ~~~~  This is where we call up our controls and check their state ~~~~ //
    
    //Input and output pointers for our buffer
	const Float32 *sourceP = inSourceP;
	Float32 *destP = inDestP;
    
    // Some math for our clipper
    float four_over_pi = 4/M_PI;
    
    // ~~~~ Make sure parameters are in bounds and assign names to our parameters so we can pass them to the DSP as requested by the UI ~~~~ //
    //Gain
    // Once per input buffer, gets the gain via the audio unit view
     Float32 gain = GetParameter(kParam_One);
    //Check bounds
     if(gain < kMinValue_ParamOne) gain = kMinValue_ParamOne;
     if(gain > kMaxValue_ParamOne) gain = kMaxValue_ParamOne;
     
  //Tone
     Float32 tone = GetParameter(kParam_Two);
     if(tone < kMinValue_ParamTwo) tone = kMinValue_ParamTwo;
     if(tone > kMaxValue_ParamTwo) tone = kMaxValue_ParamTwo;
    
    //Vol
    Float32 vol = GetParameter(kParam_Three);
    if(vol < kMinValue_ParamThree) vol = kMinValue_ParamThree;
    if(vol > kMaxValue_ParamThree) vol = kMaxValue_ParamThree;
    
  // ~~~~ Make sure we are not doing unneccessary math ~~~~ //
     //Calculate parameters only if paramter has changed
     if( gain != mLastGain || tone != mLastTone || vol != mLastVol)
     {
    //Set out tone filter and tie it to our 'tone' control
    Tone->setBiquad(bq_type_bandpass, tone / GetSampleRate(), 0.5, 0.0);
     
    //Check param status
         mLastGain = gain;
         mLastTone = tone;
         mLastVol = vol;
     }
    
// ~~~~ This is where you want to process data to produce an effect ~~~~ //
    
    /* 
     * The filter functions will slow down the code signifigantly since they are not optimized.
     *  I recommend optimizing the math for your target, and using a more efficient process function,
     *  like Apple's vDSP_deq_22 (for targets <10.9) or vDSP_biquad (for targets >10.9).  See
     *  my blog at alexkenis.wordpress.com for usage details and an example project.
     */

    //PROCESS: Pre-gain biquad bandpass filter
    for (int idx = 0; idx < inFramesToProcess; idx++) {
        destP[idx] = TSBandpass->process(sourceP[idx]);
    }
		
        //PROCESS: Gain using vDSP library Multiply
        vDSP_vsmul(sourceP, 1, &gain, destP, 1, inFramesToProcess);
        
            //PROCESS: Clip - using vDSP library ATAN
            vvatanf(destP, sourceP,(const int*)&inFramesToProcess);
            vDSP_vsmul(destP, 1, (const float*)&four_over_pi, destP, 1, inFramesToProcess);
    
                //PROCESS: Output Tone biquad bandpass filter
                for (int idx = 0; idx < inFramesToProcess; idx++) {
                        destP[idx] = Tone->process(sourceP[idx]);
                    }
                    //PROCESS: Volume using vDSP library Multiply
                    vDSP_vsmul(sourceP, 1, &vol, destP, 1, inFramesToProcess);
    
}
