/*
 
     File: PPPPPPPP.h
 Abstract: Audio Unit class definition.
  Version: 1.0.1
    Template by Alex Kenis 2014
 
*/

#include "AUEffectBase.h"                       // Apple's effect base
#include "PPPPPPPPVersion.h"                    // our project's version file
#include <Accelerate/Accelerate.h>              // Apple's vDSP framework for our DSP
#include "Biquad.h"                             // biquad filter class for our tone control
#include <AudioToolbox/AudioUnitUtilities.h>    // Apple's misc audio utilities
#include <cmath>                                // C language basic math functions for 'pi' etc

#if AU_DEBUG_DISPATCHER
	#include "AUDebugDispatcher.h" // dispatcher for debugger
#endif


#ifndef __PPPPPPPP_h__
#define __PPPPPPPP_h__

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	AU Kernel
//      Here we pull up the basic thing's we'll need from the stock Apple AUKernelBase
//      to create a kernel to hold our DSP, we are grabbing 'process' and 'reset'
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

class PPPPPPPPKernel : public AUKernelBase		// create a kernel to hold our DSP
{
public:
    PPPPPPPPKernel(AUEffectBase *inAudioUnit );
	virtual ~PPPPPPPPKernel();
    
    // *Required* overides for the process method for this effect
    // processes one channel of interleaved samples
    virtual void 		Process(	const Float32 	*inSourceP,
                                Float32		 	*inDestP,
                                UInt32 			inFramesToProcess,
                                UInt32			inNumChannels,
                                bool			&ioSilence);
    
    virtual void		Reset();
    
private:

  // ~~~~  Here we'll add a few things that we will need later ~~~~ //
    // We'll use these to reset our parameters and chech their state
    double mLastGain, mLastTone, mLastVol;
    
    // get samplerate from host for our filter math
    double sampleRate = GetSampleRate();
    
    // Set up a few filters from our biquad class that we'll use later in our process method
    //  -The first is the pre-gain bandpass filter... we set all it's parameters here
    //  -The second is out post-gain tone filter, which we will set up later so we can hook it up
    //      to a control in the UI
    Biquad *TSBandpass = new Biquad(bq_type_bandpass, 777 / sampleRate, 0.5, 0.0);
    Biquad *Tone = new Biquad();
};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	AU Effect
//      Here we pull up the basic thing's we'll need from the stock Apple AUEffectBase.
//      We are grabbing:
//          GetParameterInfo for our Gain/Tone/Vol controls
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#pragma mark ____PPPPPPPP
class PPPPPPPP : public AUEffectBase
{
public:
	PPPPPPPP(AudioUnit component);
#if AU_DEBUG_DISPATCHER
	virtual ~PPPPPPPP () { delete mDebugDispatcher; }
#endif
    // Initialize function
    virtual OSStatus			Initialize();
	
    // set up the kernel
	virtual AUKernelBase *		NewKernel() { return new PPPPPPPPKernel(this); }
	
    
	virtual	OSStatus			GetParameterValueStrings(AudioUnitScope			inScope,
														 AudioUnitParameterID		inParameterID,
														 CFArrayRef *			outStrings);
    
    // Get up info on our cocoa UI
	virtual	OSStatus			GetParameterInfo(AudioUnitScope			inScope,
												 AudioUnitParameterID	inParameterID,
												 AudioUnitParameterInfo	&outParameterInfo);
    
    // Get up info on our cocoa UI
	virtual OSStatus			GetPropertyInfo(AudioUnitPropertyID		inID,
												AudioUnitScope			inScope,
												AudioUnitElement		inElement,
												UInt32 &			outDataSize,
												Boolean	&			outWritable );
	
    
	virtual OSStatus			GetProperty(AudioUnitPropertyID inID,
											AudioUnitScope 		inScope,
											AudioUnitElement 		inElement,
											void *			outData);
	
    // We have no tail to report to the host.  (An effect like a reverb will)
 	virtual	bool				SupportsTail () { return false; }
    
    // We have no latency to report
    virtual Float64				GetLatency() {return 0.0;}
    
	// Grab the version information from our PPPPPPPPVersion.h file
	/*! @method Version */
	virtual OSStatus		Version() { return kPPPPPPPPVersion; }
	
    
	
protected: // Nothing here for this plugin

};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// List our control parameters, set their min/max/default values and their names
//      The CFSRT("****") value is the name of the control that will appear next to
//      it in the generic view.
// The max/min/default values occur in 3 places: PPPPPPPP.h, PPPPPPPP.cpp and PPPPPPPP_CocoaView.xib.
//      If you change them, you'll have to make the corresponding changes in all 3 places
//      to avoid conflicts when the host loads them.
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#pragma mark ____PPPPPPPP Parameters

// Gain
static CFStringRef kParameterOneName = CFSTR("Gain");
const float kMinValue_ParamOne = 1;
const float kMaxValue_ParamOne = 100;
static const float kDefaultValue_ParamOne = 25;

// Tone
static CFStringRef kParameterTwoName = CFSTR("Tone");
static const float kDefaultValue_ParamTwo = 600;
const float kMinValue_ParamTwo = 500;
const float kMaxValue_ParamTwo = 999;

// Volume
static CFStringRef kParameterThreeName = CFSTR("Vol");
static const float kDefaultValue_ParamThree = 0.25;
const float kMinValue_ParamThree = 0.01;
const float kMaxValue_ParamThree = 1;

// Name and define constants to identify the parameters for the AU (we'll use this list later in the GUI as well), as well as the total number of parameters
enum {
	kParam_One =0,
    kParam_Two =1,
    kParam_Three =2,
	kNumberOfParameters=3
};

#endif
