/*
 
     File: PPPPPPPP_CocoaView.h
 Abstract: Audio Unit Cocoa View Factory definition.
  Version: 1.0.1
    Template by Alex Kenis 2014
*/

#import <Cocoa/Cocoa.h>
#import <AudioUnit/AudioUnit.h>
#import <AudioToolbox/AudioToolbox.h>

/************************************************************************************************************/
/* NOTE: It is important to rename ALL ui classes when using the Xcode Audio Unit with Cocoa View template	*/
/*		 Cocoa has a flat namespace, and if you use the default filenames, it is possible that you will		*/
/*		 get a namespace collision with classes from the cocoa view of a previously loaded audio unit.		*/
/*		 We recommend that you use a unique prefix that includes the manufacturer name and unit name on		*/
/*		 all objective-C source files. You may use an underscore in your name, but please refrain from		*/
/*		 starting your class name with an undescore as these names are reserved for Apple.					*/
/*  Example  : AppleDemoFilter_UIView AppleDemoFilter_ViewFactory											*/
/************************************************************************************************************/

// This is the main header for your custom cocoa UI
// Here we'll set up our elements and custom subclasses
//      that we can pull up later.

// Set up our sliders here and tie them into the user input changes
@interface PPPPPPPP_GestureSlider : NSSlider {}
@end

@interface PPPPPPPP_CocoaView : NSView
{
    @private
    // IB Members: our sliders
    
    //Gain
    IBOutlet PPPPPPPP_GestureSlider * uiParam1Slider; // the gain knob
    IBOutlet NSTextField *			uiParam1TextField; // the gain value readout
    
    //Tone
    IBOutlet PPPPPPPP_GestureSlider * uiParam2Slider; // the tone knob
    IBOutlet NSTextField *			uiParam2TextField; // the tone value readout
    
    //Volume
    IBOutlet PPPPPPPP_GestureSlider * uiParam3Slider; // the volume knob
    IBOutlet NSTextField *			uiParam3TextField; // the volume value readout
	
    // Other Members
    AudioUnit 				mAU;
	AudioUnitParameter		mParameter[3]; // set to number of parameters... we have 3
    AUParameterListenerRef	mParameterListener;
}

#pragma mark ____ PUBLIC FUNCTIONS ____
- (void)setAU:(AudioUnit)inAU;

// Setup senders for each control slider
#pragma mark ____ INTERFACE ACTIONS ____
- (IBAction)iaParam1Changed:(id)sender; // gain
- (IBAction)iaParam2Changed:(id)sender; // tone
- (IBAction)iaParam3Changed:(id)sender; // vol

#pragma mark ____ PRIVATE FUNCTIONS
- (void)_synchronizeUIWithParameterValues;
- (void)_addListeners;
- (void)_removeListeners;

#pragma mark ____ LISTENER CALLBACK DISPATCHEE ____
- (void)_parameterListener:(void *)inObject parameter:(const AudioUnitParameter *)inParameter value:(Float32)inValue;

@end
